function SMCC=MICCluster(LM,EM,eM,DM,tSMCC)
%cluster MIC into tree and find its maximal complete cluter
%LM: list of MIC's; EM: corresponding maximal tolerances of each MIC; 
%DM: distances (minimal tolerance) between MIC's
%SMCC: each mumeber of SMCC is a list of MICs, expressed as point indices in P,  
% constructing an incomplete symmetry 

%array value: parent (T), vertex number(V), edge number(E), complete or not(C)
%array index: MIC cluster 
nM=size(LM,2);
T=[]; V=[]; E=[]; C=[];
em=eM; Em=EM; 
for i=1:nM T(i)=0; V(i)=1; E(i)=0; C(i)=0; end % C(i) is set for 0 for simplicity 
[sDM,idx]=sort(DM);
% exploring distance from min to max
for k=1:size(DM,2)
    kd=idx(k);
    e=DM(kd);
    if(isinf(e)) k=k;
        break; end
    [i0 j0]=GetMatIJ(kd,nM);
    if(i0>=j0) continue; end   
    ip=GetParent(i0,T);  
    jp=GetParent(j0,T);        
    et=max([e,em(ip),em(jp)]);
    Et=min([Em(ip),Em(jp)]);
    if(et>Et) continue; end
    
    if(ip==jp)% two in a single cluster 
        E(ip)=E(ip)+1; 
        if(E(ip)==V(ip)*(V(ip)-1)/2) 
            C(ip)=1;
            em(ip)=et;
            Em(ip)=Et;
        end
    else % two in different clusters 
        nT=size(T,2)+1;
        T(nT)=0;
        T(ip)=nT;
        T(jp)=nT;
        V(nT)=V(ip)+V(jp);
        E(nT)=E(ip)+E(jp)+1;
        em(nT)=et;
        Em(nT)=Et;
        if(E(nT)==V(nT)*(V(nT)-1)/2) C(nT)=1;
        else C(nT)=0;
        end
    end
end
s=size(tSMCC,2);
for i=1:size(C,2)
    if(C(i) & ~hasCParent(i,C,T))
        if(i<=nM) Ch(1)=i; 
        else
            Ch=GetChildren(i,nM,T);
        end
        MM=[]; mm=0;        
        for j=1:size(Ch,2)
            mm=mm+1;
            MM{mm}=LM{Ch(j)}; 
        end                    
        s=s+1;
        tSMCC{s}=MM;
    end
end
SMCC=tSMCC;


function t=hasCParent(i,C,T)
%check whether the complete node is the highest one 
t=false; 
while(T(i)) 
    i=T(i); 
    if(C(i)==1) t=true; break; end
end

function Ch=GetChildren(p,nM,T)
%p, current node
%nM, all the leaf number;
%T: the tree; 
Ch=[];
s=0;
for i=1:nM
    i0=i;
    while(T(i0))
        i0=T(i0);
        if(i0==p) s=s+1; Ch(s)=i; break; end        
    end
end

% if a single cluster, its parent is itself 
function ip=GetParent(i0,T)
while(T(i0)) 
    i0=T(i0);
end
ip=i0;