function DM=MICDist(LM,EM,eM,D,P,tag)% common points have been checked to be right
%LM: list of MIC, which are represented by point order a point set P
%nP: the point number of P; 
%D: the distances between pairwise distinct point in P.
%tag: symmetry type 
%if not valid, the distance is set as infinity
nP=size(P,2);
DM=[];
nM=size(LM,2);
for i=1:nM
    iM=LM{i};
    iEM=EM(i);
    ieM=eM(i); 
    for j=i+1:nM
        jM=LM{j};        
        jEM=EM(j);
        jeM=eM(j);
        tEMax=min([iEM,jEM]);
        tEMin=max([ieM,jeM]);        
        if(~isempty(intersect(iM,jM))|tEMax<tEMin) 
            d=Inf;   
            %if not consistent or have common points
        else             
            jM=VerifyOrder(iM,jM,P,tag); %make the point direction consistent
            switch tag
                case Inf %translation               
                    if(size(iM,2)<3) d=Inf; end
                   d=RotDis(iM,jM,D,nP,100);           
                case 1%reflection
                   d=MirrorDis(iM,jM,D,P,tag);              
                case 2%2-fold
                   d=MirrorDis(iM,jM,D,P,tag);
                otherwise%n-fold
                    if(size(iM,2)<3) d=Inf; end
                   d=RotDis(iM,jM,D,nP,tag);
            end        
        end
        tEMin=max(tEMin,d);
        if(tEMin>tEMax) d=Inf; end % if not consistent
        DM((i-1)*nM+j)=d;      
        DM((j-1)*nM+i)=d; 
    end
    DM((i-1)*nM+i)=Inf;      
end
%total number of finite DM: nM*(nM-1)/2* 3* 2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function d=MirrorDis(M1,M2,EM1,EM2,D,nP)
function d=MirrorDis(M1,M2,D,P,tag)
%if unvalid MIC (common points or Emin>Emax), d=Inf
% M1 and M2 have been in proper order 
% M1, M2: two gives MIC's, represented by point label in a point set P 
% D: the distances between pairwise distinct point in P.  
% nP: the point number of P;     
nP=size(P,2);
d11=D( (M1(1)-1)*nP+M2(1) );   
d22=D( (M1(2)-1)*nP+M2(2) );
d12=D( (M1(1)-1)*nP+M2(2) );
d21=D( (M1(2)-1)*nP+M2(1) );
d=max(abs(d11-d22), abs(d12-d21)); 
A=P(:,M1(1));
B=P(:,M1(2));
C=P(:,M2(1));
D=P(:,M2(2));
if((tag==1&isIntersect(A,B,C,D)) | (tag==2&~isIntersect(A,B,C,D))) d=Inf; end  
% d>1 is arbitrary decision to reducing number


%M1,M2 have the same orientations 
function e=RotDis(M1,M2,D,nP,c)
e=0;
nM1=size(M1,2);
nM2=size(M2,2);
for r=0:min([nM1-1,nM2-1,floor(c/2)])% step length
    dd=100000.;
    DD=0;
    for i=1:min([nM1,nM2]);
        i0=mod(i,c); if(i0==0) i0=c; end
        ir0=mod(i+r,c); if(ir0==0) ir0=c; end        
        if(i0>nM1|ir0>nM2) continue; end      
        d1=D((M1(i0)-1)*nP+M2(ir0));
        dd=min([dd,d1]);
        DD=max([DD,d1]);
    end
    e=max(e,DD-dd);
end 


function NS2=VerifyOrder(S1,S2,P,C)
ns1=size(S1,2);
ns2=size(S2,2);
P11=P(:,S1(1));
P12=P(:,S1(2));
P1e=P(:,S1(ns1));
P21=P(:,S2(1));
P22=P(:,S2(2));
P2e=P(:,S2(ns2));
if(size(S1,2)>=3&size(S2,2)>=3)
    P13=P(:,S1(3));
    P23=P(:,S2(3));
end
if(isinf(C)|C==1|C==2)
    if(C==1|C==2)                    
        d1=P13-P11;                
        d2=P23-P21;
    else
        d1=P12-P11;
        d2=P22-P21;
    end
else 
    if(ns1>=6&ns2>=6)
        P15=P(:,S1(5));
        d11=P11-P13;
        d12=P15-P13;           
        P25=P(:,S2(5));
        d21=P21-P23;
        d22=P25-P23;
    else
        d11=P11-P12;
        d12=P13-P12;
        d21=P21-P22;
        d22=P23-P22;
    end
    d1=cross(d11,d12);
    d2=cross(d21,d22);    
end
angle=dot(d1,d2);
if(angle>=0|abs(angle)<0.0001) NS2=S2;
else  NS2=S2(ns2:-1:1);
end


