function [Pe,cc,em,eM,dd,DD,D234]=PointExpansion(S0,c,eMin,eMax,dr,Dr,P,D,NrD,isCst,isT,D123,Ms)
%Pe:   the returned point index; if Pe=-1, no expansion point found 
%expansion directions have to be controlled as well: 1. along the line; 
% 2. rotation; 3. glide
%isCst: consistent or not consistent rotation direction.
%isT,D123,Ms, for translation condition determination
Pe=-1; cc=c; 
em=eMax; eM=eMax;
dd=dr; DD=Dr; 
nP=size(P,2);  ns0=size(S0,2);
D234=D123;
for k=1:nP
    if(ismember([k],S0)) continue; end
    d1=D((S0(ns0)-1)*nP+k);
    if(abs(d1-DD(1))>eMax) continue; end    
%    if((ns0==3) | isT)     
        [isc,D234]=isConsistent(D123,P(:,S0(ns0-1)),P(:,S0(ns0)),P(:,k),isT,Ms);
        if(xor(isc,isCst) ) continue; end %this condition also speficifies no translation in zig-zag line case
%    end    
    [c0,em0,eM0,dd0,DD0]=ExpansionError(S0,c,eMin,eMax,dr,Dr,k,P,D);        
    if(em0<0) continue; end
    if(em0<em)
        Pe=k;    cc=c0; 
        em=em0;  eM=eM0;
        dd=dd0;  DD=DD0;
    end
end
if(Pe>0)
    eM=min([eM,NrD(Pe,1)]);
    if(eM<em) Pe=-1; end
end


function [c,D234]=isConsistent(D123,P2,P3,P4,isT,Ms)
% a simple condition is applied here for angle checking. 
D234=cross(P2-P3,P4-P3);
if(isT) 
    if(dot(D234,D234)<Ms) c=true; else c=false; end
else
    if(dot(D123,D234)>0) c=true; else c=false; end
end



function [cc,em,eM,dd,DD]=ExpansionError(S0,c,eMin,eMax,dr,Dr,Pk,P,D)
% expansion error of a point Pk w.r.t. a seed set S0 with info. eMin, eMax,
% dr, Dr,  P,D
% if em=-1 not a valid expansion point
nP=size(P,2); s=size(S0,2); ng=size(dr,2);
em=eMin; eM=eMax; dd=dr; DD=Dr;
cc=c;
for r=1:ng
     de(r)=dr(r)-eMax;    De(r)=Dr(r)+eMax;
end
for i=1:s  
    L(i)=D((S0(s-i+1)-1)*nP+Pk);  
end
for r=1:ng
     if( L(r)<de(r)|L(r)>De(r) ) em=-1; return; end
     if( c>0 & r>=c-s ) 
         if ( L(c-r)<de(r)|L(c-r)>De(r) ) em=-1; return; end
         dd(r)=min([dr(r),L(r),L(c-r)]); DD(r)=max([Dr(r),L(r),L(c-r)]);
     else 
         dd(r)=min([dr(r),L(r)]); DD(r)=max([Dr(r),L(r)]); 
     end
     
     e1=DD(r)-dd(r);
     if(e1>em) em=e1; end
     if(r>1) E1=dd(r)-DD(r-1);
         if(E1<eM) eM=E1; end
     end
     if(eM<em)  em=-1; return; end
 end
 %we are still looking for distance difference em,eM,dd,DD

 %check the membership of L(ns0)
 if(c==0) 
     if(ng~=s-1) 
         fprintf('Not consistent with group number: %d,%d.\n',ng,s); 
         em=-1; return; 
     end
     %interval 1: none
     if(L(s)<de(s-2)) em=-1; return; end
     %interval 2: s-2
     if(L(s)>=de(s-2)&L(s)<dd(s-2)) 
         e=max([DD(s-2)-L(s), em]);
         E=eM;
         if(s>3)  E=min([eM, L(s)-DD(s-3)]); end
         if(e<E) cc=2*(s-1); em=e; eM=E; dd(s-2)=L(s); DD(s-2)=DD(s-2);
         else em=-1; end
         return;
     end    
     %interval 3: s-2
     if(L(s)>=dd(s-2)&L(s)<=DD(s-2)) cc=2*(s-1); return; end
     %interval 4: 
     if(L(s)>DD(s-2)&L(s)<dd(s-1))
         %s-2
         e1=max([em, L(s)-dd(s-2)]);
         E1=min([eM, dd(s-1)-L(s)]);
         %s-1
         e2=max([em, DD(s-1)-L(s)]);
         E2=min([eM, L(s)-DD(s-2)]);
         if( e1<E1 & (e2>=E2 | e1<e2)  ) %s-2
             cc=2*(s-1);
             em=e1;
             eM=E1;
             dd(s-2)=dd(s-2);
             DD(s-2)=L(s);
             return;
         end
         if( e2<E2 & (e1>=E1 | e2<e1)  ) %s-1
             cc=2*s-1;
             em=e2;
             eM=E2;
             dd(s-1)=L(s);
             DD(s-1)=DD(s-1);
             return;
         end
         em=-1;  return;
     end
     %interval 5: s-1
     if(L(s)>=dd(s-1)&L(s)<=DD(s-1)) cc=2*s-1; return; end

     %interval 6: s-1
     if(L(s)>DD(s-1)&L(s)<DD(s-1)+em)
         e=max([em,L(s)-dd(s-1)]);
         if(e<eM) 
             cc=2*s-1;
             em=e; 
             eM=eM;
             dd(s-1)=dd(s-1);
             DD(s-1)=L(s);
         else em=-1; 
         end
         return;
     end
     %interval 7: s
     if(L(s)>=DD(s-1)+em)  
         cc=0;
         em=em;
         eM=min([eM,L(s)-DD(s-1)]);
         dd(s)=L(s);
         DD(s)=L(s);
         return; 
     end
 end